#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "owl.h"


void owl_process_command(char *buff) {
  char **argv;
  int argc;
  char *tmpbuff;

  tmpbuff=strdup(buff);
  argv=owl_parseline(buff, &argc);
  
  if (argc < 1) return;

  if (!strcmp(argv[0], "")) {
    /* do nothing */
  } else if (!strcmp(argv[0], "quit") ||
	     !strcmp(argv[0], "exit") ||
	     !strcmp(argv[0], "q")) {
    owl_command_quit();
  } else if (!strcmp(argv[0], "info")) {
    owl_command_info();
  } else if (!strcmp(argv[0], "help")) {
    owl_command_help();
  } else if (!strcmp(argv[0], "version")) {
    owl_command_version();
  } else if (!strcmp(argv[0], "next")) {
    owl_command_next();
  } else if (!strcmp(argv[0], "prev")) {
    owl_command_prev();
  } else if (!strcmp(argv[0], "next-notdel")) {
    owl_command_next_notdeleted();
  } else if (!strcmp(argv[0], "prev-notdel")) {
    owl_command_prev_notdeleted();
  } else if (!strcmp(argv[0], "expunge")) {
    owl_command_expunge();
  } else if (!strcmp(argv[0], "last")) {
    owl_command_last();
  } else if (!strcmp(argv[0], "first")) {
    owl_command_first();
  } else if (!strcmp(argv[0], "resize")) {
    owl_command_resize();
  } else if (!strcmp(argv[0], "suspend")) {
    owl_command_suspend();
  } else if (!strcmp(argv[0], "exec")) {
    owl_command_exec(argc, argv);
  } else if (!strcmp(argv[0], "unsub")) {
    owl_command_unsub();
  } else if (!strcmp(argv[0], "load-subs")) {
    owl_command_loadsubs(argc, argv);
  } else if (!strcmp(argv[0], "zlog")) {
    owl_command_zlog(argc, argv);
  } else if (!strcmp(argv[0], "debug")) {
    owl_command_debug(argc, argv);
  } else if (!strcmp(argv[0], "set")) {
    owl_command_set(argc, argv);
  } else if (!strcmp(argv[0], "print")) {
    owl_command_print(argc, argv);
  } else if (!strcmp(argv[0], "ktest")) {
    owl_command_ktest();
  } else if (!strcmp(argv[0], "zwrite")) {
    if (argc < 2) {
      owl_function_makemsg("Not enough arguments to the zwrite command.");
    } else {
      owl_command_zwrite_setup(tmpbuff);
      owl_global_set_buffercommand(&g, tmpbuff);
    }
  } else if (!strcmp(argv[0], "reply")) {
    char buff[1024];
    owl_message *m;

    if (owl_messagelist_get_size(owl_global_get_msglist(&g))==0) {
      owl_function_makemsg("No message selected");
    } else {
      char *to;
      
      m=owl_messagelist_get_element(owl_global_get_msglist(&g), owl_global_get_curmsg(&g));

      if (owl_message_is_admin(m)) {
	owl_function_makemsg("You can not reply to an admin message");
      } else {
	to=owl_message_get_recipient(m);
	if (!strcmp(to, "") || !strcmp(to, "*")) {
	  to="";
	} else {
	  to=owl_message_get_sender(m);
	}
	
	sprintf(buff, "zwrite -c %s -i %s %s", 
		owl_message_get_class(m),
		owl_message_get_instance(m),
		to);
	
	owl_command_zwrite_setup(buff);
	owl_global_set_buffercommand(&g, buff);
      }
    }
  } else if (!strcmp(argv[0], "zaway")) {
    owl_command_zaway(argc, argv);
  } else {
    char tmpbuff[1024];
    sprintf(tmpbuff, "Unknown command '%s'.", buff);
    owl_function_makemsg(tmpbuff);
  }
  owl_parsefree(argv, argc);
  free(tmpbuff);
  sepbar(NULL);
}


void owl_command_info() {
  owl_message *m;
  ZNotice_t n;
  char buff[2048], tmpbuff[1024];
  char *ptr;
  owl_popwin *pw;
  int i, j, fields, len;
  
  pw=owl_global_get_popwin(&g);
  
  owl_popwin_up(pw);
  owl_popwin_set_handler(pw, pophandler_quit);

  if (owl_messagelist_get_size(owl_global_get_msglist(&g))==0) {
    owl_popwin_display_text(pw, "No message selected\n\nPress 'q' to exit.");
    return;
  }

  m=owl_messagelist_get_element(owl_global_get_msglist(&g), owl_global_get_curmsg(&g));
  if (!owl_message_is_zephyr(m)) {
      strcpy(buff, "No zephyr info for this message\n\nPress 'q' to exit.");
      owl_popwin_display_text(pw, buff);
      return;
  }

  n=owl_message_get_notice(m);

  sprintf(buff,   "Class     : %s\n", n.z_class);
  sprintf(buff, "%sInstance  : %s\n", buff, n.z_class_inst);
  sprintf(buff, "%sSender    : %s\n", buff, n.z_sender);
  sprintf(buff, "%sRecip     : %s\n", buff, n.z_recipient);
  sprintf(buff, "%sOpcode    : %s\n", buff, n.z_opcode);
  strcat(buff,    "Kind      : ");
  if (n.z_kind==UNSAFE) {
    strcat(buff, "UNSAFE\n");
  } else if (n.z_kind==UNACKED) {
    strcat(buff, "UNACKED\n");
  } else if (n.z_kind==ACKED) {
    strcat(buff, "ACKED\n");
  } else if (n.z_kind==HMACK) {
    strcat(buff, "HMACK\n");
  } else if (n.z_kind==HMCTL) {
    strcat(buff, "HMCTL\n");
  } else if (n.z_kind==SERVACK) {
    strcat(buff, "SERVACK\n");
  } else if (n.z_kind==SERVNAK) {
    strcat(buff, "SERVNAK\n");
  } else if (n.z_kind==CLIENTACK) {
    strcat(buff, "CLIENTACK\n");
  } else if (n.z_kind==STAT) {
    strcat(buff, "STAT\n");
  } else {
    strcat(buff, "ILLEGAL VALUE\n");
  }
  sprintf(buff, "%sTime      : %s\n", buff, owl_message_get_timestr(m));
  sprintf(buff, "%sHost      : %s\n", buff, owl_message_get_hostname(m));
  sprintf(buff, "%sPort      : %i\n", buff, n.z_port);
  strcat(buff,    "Auth      : ");
  if (n.z_auth == ZAUTH_FAILED) {
    strcat(buff, "FAILED\n");
  } else if (n.z_auth == ZAUTH_NO) {
    strcat(buff, "NO\n");
  } else if (n.z_auth == ZAUTH_YES) {
    strcat(buff, "YES\n");
  } else {
    sprintf(buff, "%sUnknown State (%i)\n", buff, n.z_auth);
  }
  sprintf(buff, "%sCheckd Ath: %i\n", buff, n.z_checked_auth);
  sprintf(buff, "%sMulti notc: %s\n", buff, n.z_multinotice);
  sprintf(buff, "%sNum other : %i\n", buff, n.z_num_other_fields);
  sprintf(buff, "%sMsg Len   : %i\n", buff, n.z_message_len);

  sprintf(buff, "%sFields    : %i\n", buff, owl_zephyr_get_num_fields(&n));

  fields=owl_zephyr_get_num_fields(&n);
  for (i=0; i<fields; i++) {
    sprintf(buff, "%sField %i   : ", buff, i+1);
    ptr=owl_zephyr_get_field(&n, i+1, &len);
    if (!ptr) break;
    strncpy(tmpbuff, ptr, len);
    tmpbuff[len]='\0';

    /* just for testing for now */
    for (j=0; j<strlen(tmpbuff); j++) {
      if (tmpbuff[j]=='\n') tmpbuff[j]='~';
      if (tmpbuff[j]=='\r') tmpbuff[j]='!';
    }

    if (strlen(tmpbuff)>30) {
      strncat(buff, tmpbuff, 30);
      strcat(buff, "...");
    } else {
      strcat(buff, tmpbuff);
    }
    strcat(buff, "\n");
  }
  sprintf(buff, "%sDefault Fm: %s\n", buff, n.z_default_format);


  strcat(buff, "\n\nPress 'q' to exit.");
	
  owl_popwin_display_text(pw, buff);
}


void owl_command_help() {
  char buff[1024];
  owl_popwin *pw;

  pw=owl_global_get_popwin(&g);
  
  owl_popwin_up(pw);
  owl_popwin_set_handler(pw, pophandler_quit);

  strcpy(buff, "owl help\n\n");
  strcat(buff, "  Keys:\n");
  strcat(buff, "    n    View next message not marked for deletion\n");
  strcat(buff, "    p    View previous message not marked for deletion\n");
  strcat(buff, "    C-n  View next message\n");
  strcat(buff, "    C-p  View previous message\n");
  strcat(buff, "    d    Mark a message as deleted\n");
  strcat(buff, "    x    Expunge deleted messages\n");
  strcat(buff, "    u    Undelete deleted messages that have not been expunged\n");
  strcat(buff, "    i    Print more information about a message\n");
  strcat(buff, "    z    Start a zwrite command\n");
  strcat(buff, "    r    Reply to the current message\n");
  strcat(buff, "    h    Print this help message\n");
  strcat(buff, "    :    Enter command mode\n");
  strcat(buff, "\n");
  strcat(buff, "  Commands:\n");
  strcat(buff, "    zwrite [-c class] [-i instance] [user]  Send a message.\n");
  strcat(buff, "    quit      Exit owl\n");
  strcat(buff, "    info      Print more information about a message\n");
  strcat(buff, "    help      Print this help message\n");
  strcat(buff, "    zlog in   Send a login notice\n");
  strcat(buff, "    zlog out  Send a logout notice\n");
  strcat(buff, "    unsub     Unsubscribe from everything\n");
  strcat(buff, "    load-subs [file] Subscribe to everything in [file]\n");
  strcat(buff, "    set bell <on|off>  Turn the terminal bell on or off\n");
  strcat(buff, "    next\n");
  strcat(buff, "    prev\n");
  strcat(buff, "    next-notdel\n");
  strcat(buff, "    prev-notdel\n");
  strcat(buff, "    delete\n");
  strcat(buff, "    expunge\n");
  strcat(buff, "    undelete\n");
  strcat(buff, "\n\n");
  strcat(buff, "Type 'q' to exit.");

  owl_popwin_display_text(pw, buff);
}

void owl_command_version() {
  char buff[1024];

  sprintf(buff, "Owl version %s", OWL_VERSION_STRING);
  owl_function_makemsg(buff);
}

void owl_command_next() {
  owl_function_nextmsg();
}

void owl_command_prev() {
  owl_function_prevmsg();
}

void owl_command_next_notdeleted() {
  owl_function_nextmsg_notdeleted();
}

void owl_command_prev_notdeleted() {
  owl_function_prevmsg_notdeleted();
}

void owl_command_delete_cur() {
  owl_function_deletecur();
}

void owl_command_undelete_curmsg() {
  owl_function_undeletecur();
}

void owl_command_expunge() {
  owl_function_expunge();
}

void owl_command_first() {
  owl_function_firstmsg();
}

void owl_command_last() {
  owl_function_lastmsg();
}

void owl_command_resize() {
  owl_global_set_resize_pending(&g);
}

void owl_command_shift_right() {
  owl_function_shift_right();
}

void owl_command_shift_left() {
  owl_function_shift_left();
}

void owl_command_unsub() {
  owl_function_unsub();
}

void owl_command_loadsubs(int argc, char **argv) {
  if (argc != 2) {
    owl_function_makemsg("Wrong number of arguments for load-subs.");
    return;
  }
  owl_function_loadsubs(argv[1]);
}

void owl_command_zwrite_setup(char *line) {
  /* owl_typwin_getmore(owl_global_get_typwin(&g)); */
  owl_editwin *e;
  char buff[1024];

  e=owl_global_get_typwin(&g);

  owl_editwin_new_style(e, OWL_EDITWIN_STYLE_MULTILINE);

  owl_function_makemsg("Type your zephyr below.  C-d to send C-c to quit.");

  owl_editwin_clear(e);
  strcpy(buff, "----> ");
  strcat(buff, line);
  strcat(buff, "\n");
  owl_editwin_set_locktext(e, buff);

  owl_global_set_typwin_active(&g);
}

void owl_command_zwrite(char *line) {
  char class[1024], inst[1024], recip[1024], buff[1024];
  char *tmpbuff;
  char **argv;
  int argc, i;

  strcpy(buff, line);

  strcpy(class, "message");
  strcpy(inst, "personal");
  strcpy(recip, "");


  argv=atokenize(buff, " \t\n\r", &argc);
  if (argc<0) {
    owl_function_makemsg("Invalid arguments");
    return;
  }

  for (i=1; i<argc; i++) {
    if (!strcmp(argv[i], "-c")) {
      strcpy(class, argv[i+1]);
      i++;
    } else if (!strcmp(argv[i], "-i")) {
      strcpy(inst, argv[i+1]);
      i++;
    } else {
      strcpy(recip, argv[i]);
      break;
    }
  }

  if (!strcasecmp(class, "message") &&
      !strcasecmp(inst, "personal") &&
      recip[0]=='\0') {
    owl_function_makemsg("You must specify a recipient");
    return;
  }

  send_zephyr("", NULL, class, inst, recip, owl_editwin_get_text(owl_global_get_typwin(&g)));

  /* display the message as an admin message in the receive window */
  if (strcmp(recip, "") && strcmp(recip, "*")) {
    tmpbuff=malloc(strlen(owl_editwin_get_text(owl_global_get_typwin(&g)))+1024);
    sprintf(tmpbuff, "Message sent to %s", recip);
    owl_function_adminmsg(tmpbuff, owl_editwin_get_text(owl_global_get_typwin(&g)));
    free(tmpbuff);
  }
  
  for (i=0; i<argc; i++) {
    free(argv[i]);
  }
  free(argv);

  if (owl_global_is_logging(&g)) {
    if (!strcasecmp(class, "message") &&
	!strcasecmp(inst, "personal")) {
      owl_log_outgoing(recip, owl_editwin_get_text(owl_global_get_typwin(&g)));
    }
  }
}


void owl_command_suspend() {
  owl_function_suspend();
}


void owl_command_zaway(int argc, char **argv) {
  char buff[1024]; /* ick */
  int i;
  
  if (argc==1) {
    owl_function_zaway_on(NULL);
    return;
  }

  if (argc==2 && !strcmp(argv[1], "on")) {
    owl_function_zaway_on(NULL);
    return;
  }

  if (argc==2 && !strcmp(argv[1], "off")) {
    owl_function_zaway_off();
    return;
  }

  strcpy(buff, "");
  for (i=1; i<argc; i++) {
    strcat(buff, argv[i]);
    if (i!=argc-1) {
      strcat(buff, " ");
    }
  }
  owl_function_zaway_on(buff);
}


void owl_command_set(int argc, char **argv) {
  char *var, *val;

  if (argc != 3) {
    owl_function_makemsg("Wrong number of arguments for set command");
    return;
  }

  var=argv[1];
  val=argv[2];

  if (!strcmp(var, "bell")) {
    if (!strcmp(val, "on")) {
      owl_global_set_bell_on(&g);
      owl_function_makemsg("Bell on");
    } else if (!strcmp(val, "off")) {
      owl_global_set_bell_off(&g);
      owl_function_makemsg("Bell off");
    } else {
      owl_function_makemsg("Unknown bell subcommand");
    }
  } else if (!strcmp(var, "zsigproc")) {
    owl_global_set_zsig_exec(&g, val);
  } else if (!strcmp(var, "startuplogin")) {
    if (!strcmp(val, "on")) {
      owl_global_set_startuplogin_on(&g);
      owl_function_makemsg("startuplogin set to on");
    } else if (!strcmp(val, "off")) {
      owl_global_set_startuplogin_off(&g);
      owl_function_makemsg("startuplogin set to off");
    } else {
      owl_function_makemsg("Unknown setting for startuplogin variable.");
    }
  } else if (!strcmp(var, "shutdownlogout")) {
    if (!strcmp(val, "on")) {
      owl_global_set_shutdownlogout_on(&g);
      owl_function_makemsg("shutdownlogout set to on");
    } else if (!strcmp(val, "off")) {
      owl_global_set_shutdownlogout_off(&g);
      owl_function_makemsg("shutdownlogout set to off");
    } else {
      owl_function_makemsg("Unknown setting for startuplogin variable.");
    }
  } else if (!strcmp(var, "logging")) {
    if (!strcmp(val, "on")) {
      owl_global_set_logging_on(&g);
    } else if (!strcmp(val, "off")) {
      owl_global_set_logging_off(&g);
    } else {
      owl_function_makemsg("Unknown setting for logging variable.");
    }
  } else {
    owl_function_makemsg("Unknown variable");
  }
}

void owl_command_print(int argc, char **argv) {
  char *variable;
    
  if (argc!=2) {
    owl_function_makemsg("Wrong number of arguments for print command");
    return;
  }

  variable=argv[1];
  
  if (!strcmp(variable, "bell")) {
    if (owl_global_is_bell(&g)) {
      owl_function_makemsg("bell = on");
    } else {
      owl_function_makemsg("bell = off");
    }
  } else if (!strcmp(variable, "startuplogin")) {
    if (owl_global_get_startuplogin(&g)) {
      owl_function_makemsg("startuplogin = on");
    } else {
      owl_function_makemsg("startuplogin = off");
    }
  } else if (!strcmp(variable, "shutdownlogout")) {
    if (owl_global_get_shutdownlogout(&g)) {
      owl_function_makemsg("shutdownlogout = on");
    } else {
      owl_function_makemsg("shutdownlogout = off");
    }
  } else {
    /* here we should print all variables in a generic message */
    owl_function_makemsg("Unknown variable");
  }
}

void owl_command_exec(int argc, char **argv) {
  char buff[1024]; /* fix */
  int i;

  if (argc<2) {
    owl_function_makemsg("Wrong number of arguments to exec command");
    return;
  }

  strcpy(buff, "");
  for (i=1; i<argc; i++) {
    strcat(buff, argv[i]);
    strcat(buff, " ");
  }
  strcat(buff, ">/dev/null 2>/dev/null");
  system(buff);
}

void owl_command_quit() {
  owl_function_quit();
}

void owl_command_debug(int argc, char **argv) {
  if (argc<2) {
    owl_function_makemsg("Need at least one argument to debug command");
    return;
  }

  if (!owl_global_is_debug(&g)) {
    owl_function_makemsg("Debugging is not turned on");
    return;
  }

  owl_function_debugmsg(argv[1]);
}

void owl_command_ktest() {
  /* owl_message *m; */

  owl_config_execute("ktest();");
  /*
  send_zephyr("", "", "message", "personal", "kretch", "foobar\nbaz\nquux\n");

  m=malloc(sizeof(owl_message));
  owl_message_create_admin(m, "ktest", "This is a test message");
  owl_messagelist_append_element(owl_global_get_msglist(&g), m);
  */
}


void owl_command_zlog(int argc, char **argv) {
  if (argc != 2) {
    owl_function_makemsg("Wrong number of arguments for zlog command");
    return;
  }

  if (!strcmp(argv[1], "in")) {
    owl_function_zlog_in();
  } else if (!strcmp(argv[1], "out")) {
    owl_function_zlog_out();
  } else {
    owl_function_makemsg("Invalid subcommand for zlog");
  }
}


void owl_command_zlog_out() {
  owl_function_zlog_out();
}
